#ifndef PARSER_H
#define PARSER_H

#include <map>
#include <fstream>
#include <sstream>

class Parser
{
public:
    template <typename Left, typename Right>
    static void parseFile(std::string const &fname, std::map<Left, Right> &map);

private:
    template <typename Left, typename Right>
    static std::pair<Left, Right> parseLine(std::string const &line);
};


template <typename Left, typename Right>
void Parser::parseFile(std::string const &fname, std::map<Left, Right> &map)
{
    std::ifstream file(fname);
    if (!file)
        throw std::string("File not found!");

    try
    {
        std::string line;
        while (std::getline(file, line) )
            if (!line.empty())
                map.insert(parseLine<Left, Right>(line));
    }
    catch (std::string const &msg)
    {
        throw msg + "; in file: " + fname;
    }
}


template <typename Left, typename Right>
std::pair<Left, Right> Parser::parseLine(std::string const &line)
{
    // A line should be of the form:
    // [string] '=' [value]

    // 1. Break up the string in the parts before and after the '='
    size_t pos = line.find_first_of('=');
    if (pos == std::string::npos)
        throw std::string("Invalid syntax: no '=' found");

    std::string split[2] = {line.substr(0, pos), line.substr(pos + 1)};

    // 2. Trim whitespace on both strings
    for (std::string &str: split)
    {
        size_t first = str.find_first_not_of(" \t");
        size_t last  = str.find_last_not_of(" \t");
        str = str.substr(first, last - first + 1);
    }
    
    // 3. Try to convert
    static std::istringstream iss;
    Left left;
    Right right;

    iss.clear();
    iss.str(split[0]);
    iss >> left;
    if (!iss)
        throw std::string("Could not convert left-hand value");
    
    iss.clear();
    iss.str(split[1]);
    iss >> right;
    if (!iss)
        throw std::string("Could not convert right-hand value");

    // 4. Return the pair
    return {left, right};
}

#endif
