#include "picom.ih"

void PiCom::loadPins(string const &pinFile)
{
    map<string, int> pinMap;
    Parser::parseFile(pinFile, pinMap);

    static string const pinNames[N_LINES - 1] = { "SRI", "SRO", 
                                                 "SLI1", "SLO1", 
                                                 "SLI2", "SLO2"};
                        
    // Fetch all pin-numbers for the fixed lines (all but
    // message-lines) and store them in d_lines
    for (int i = 0; i != N_LINES - 1; ++i)
    {
        auto it = pinMap.find(pinNames[i]);
        if (it == pinMap.end())
            throw string("Error reading pin-file: ") + pinNames[i] + string(" missing");

        d_lines[i] = it->second;
        pinMap.erase(it);
    }


    // Only the message line information should still be in the pinmap
    // now. Check and store it in a vector that matches the
    // message-line number (1, 2, ... , N) to the pin-number
    vector<pair<int, int>> ml;

    for (auto pr: pinMap)
    {
        if (pr.first.length() <= 2)
            throw string("Error reading pin-file: unknown pin ") + pr.first;

        string str = pr.first.substr(0, 2);
        if (str == "ML")
        {
            string num = pr.first.substr(2);
            int i = atoi(num.c_str());
            if (i < 1)
                throw string("Error reading pin-file: unknown pin ") + pr.first;

            ml.push_back({i, pr.second});
        }
        else
            throw string("Error reading pin-file: unknown pin ") + pr.first;
    }

    if (ml.size() == 0)
        throw string("Error reading pin-file: you must connect at least 1 ML channel");

    d_lines[ML].setChannels(ml.size());

    // Check if the message-lines are numbered correctly
    set<int> check;
    for (auto pr: ml)
    {
        int channel = pr.first;
        int pin = pr.second;

        if (channel > static_cast<int>(ml.size()))
            throw string("ML channel numbers are 1-N");
        check.insert(pr.first);

        d_lines[ML][channel - 1] = pin;
    }

    if (check.size() != ml.size())
        throw string("Duplicate channels in MLI");
}
