/* @(#)picom.h
 */

#ifndef _PICOM_H
#define _PICOM_H 1

#include <vector>
#include <string>
#include "exception.h"

class PiCom
{
    // Exceptions
    struct TimeOut {};
    struct PinFileError {};
    struct LineError {};

    // A line, potentially containing multiple channels
    class Line: private std::vector<int>
    {
    public:
        explicit Line(int n);

        using std::vector<int>::size;
        using std::vector<int>::begin;
        using std::vector<int>::end;

        void setChannels(int n);
        int &operator[] (int i);
        int operator[] (int i) const;
        operator int();
        Line &operator=(int i);
    };

    // data-members
    std::vector<Line> d_lines;
    int d_syncLineSelect;
    bool d_syncVal[2];
    
    volatile bool d_running = false;
    volatile bool d_sending = false;
    volatile bool d_receiving = false;

    static int const TIMEOUT = 2;

public:
    explicit PiCom(std::string const &pinFile);
    virtual ~PiCom() = default;

    enum LineName
    {
        SRI, SRO,   // Send-Receive In/Out
        SLI1, SLO1, // Sync-Line In/Out
	SLI2, SLO2,
        ML,         // Message-Line In/Out
        N_LINES
    };

    void send(std::vector<bool> const &bitstream);
    std::vector<bool> listen();
    void reset();

protected:
    void sync(int num = 1, int timeout = TIMEOUT);
    bool wait(int line, int val = 1, int timeout = TIMEOUT);
    bool wait(int line, int channel, int val, int timeout);
    size_t channels(int line) const;

    bool read(int line);
    bool read(int line, int channel);
    void write(int line, bool value);
    void write(int line, int channel, bool value);
    void mode(int line, int mode);
    void mode(int line, int channel, int mode);
    void pull(int line, int mode);
    void pull(int line, int channel, int mode);

private:
    void loadPins(std::string const &pinFile);

};

inline size_t PiCom::channels(int line) const
{
    return d_lines[line].size();
}

#endif /* _PICOM_H */

